﻿using System;
using System.Threading.Tasks;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Input;

namespace EventsSample
{
    public sealed partial class MainPage : Page
    {        
        private const int msDelayTime = 500;

        public event EventHandler<RandomNumberEventArgs> RandomNumberGenerated = delegate { };
        
        public MainPage()
        {
            InitializeComponent();

            RandomNumberGenerated += MainPage_RandomNumberGenerated;            
        }

        private async void MainPage_RandomNumberGenerated(object sender, RandomNumberEventArgs e)
        {
            await Dispatcher.RunIdleAsync((a) => { IoTListBox.Items.Add(e.Value); });
        }

        private void RaiseCustomEventButton_Click(object sender, RoutedEventArgs e)
        {
            Task.Run(() => {
                Task.Delay(msDelayTime).Wait();

                //if (RandomNumberGenerated != null)
                //{
                //    RandomNumberGenerated(this, new RandomNumberEventArgs());
                //}

                // or: 
                // RandomNumberGenerated?.Invoke(this, new RandomNumberEventArgs());

                RandomNumberGenerated(this, new RandomNumberEventArgs());
            });
        }

        private void ClearButton_Click(object sender, RoutedEventArgs e)
        {
            IoTListBox.Items.Clear();            
        }

        private void IoTTextBox_KeyUp(object sender, KeyRoutedEventArgs e)
        {            
            IoTListBox.Items.Add(e.Key.ToString());
        }        

        private void KeyUpEventActiveCheckBox_Checked(object sender, RoutedEventArgs e)
        {
            CheckBox checkBox = sender as CheckBox;

            bool isChecked = IsCheckBoxChecked(checkBox);

            if (isChecked)
            {
                IoTTextBox.KeyUp += IoTTextBox_KeyUp;
            }
            else
            {
                IoTTextBox.KeyUp -= IoTTextBox_KeyUp;
            }
        }

        private bool IsCheckBoxChecked(CheckBox checkBox)
        {
            bool isChecked = false;

            if (checkBox != null)
            {
                if (checkBox.IsChecked.HasValue)
                {
                    isChecked = checkBox.IsChecked.Value;
                }
            }

            return isChecked;
        }

        private void Grid_Tapped(object sender, TappedRoutedEventArgs e)
        {
            DisplayEventRoute(sender, e.OriginalSource);
        }

        private void AppBarButton_Tapped(object sender, TappedRoutedEventArgs e)
        {
            IoTListBox.Items.Add("AppBarButton tapped event");
            e.Handled = true;
        }

        private void DisplayEventRoute(object sender, object originalSource)
        {
            string routeString = string.Empty;

            routeString = "Sender: " + GetControlTypeName(sender);
            routeString += ", original source: " + GetControlTypeName(originalSource);

            IoTListBox.Items.Add(routeString);
        }

        private string GetControlTypeName(object control)
        {
            string typeName = "Unknown";

            if(control != null)
            {
                typeName = control.GetType().Name;
            }

            return typeName;
        }        
    }
}
