﻿using System.ComponentModel;
using System.Runtime.CompilerServices;
using Windows.UI.Xaml.Controls;

namespace BinaryRepresentation
{
    public sealed partial class MainPage : Page, INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged = delegate { };

        public short BitToSet
        {
            get { return bitToSet; }
            set { SetBit(value); }            
        }

        public short BitToClear
        {
            get { return bitToClear; }
            set { ClearBit(value); }            
        }

        public short InputValue
        {
            get { return inputValue; }
            set { OutputValue = inputValue = value; }
        }

        public short OutputValue
        {
            get { return outputValue; }
            set
            {
                outputValue = value;
                OnPropertyChanged();
            }
        }

        private const int shortBitLength = 16;

        private short bitToSet;
        private short bitToClear;
        private short inputValue;
        private short outputValue;

        public MainPage()
        {
            InitializeComponent();
        }

        private void OnPropertyChanged([CallerMemberName] string propertyName = "")
        {
            PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

        private void SetBit(short position)
        {
            if(position >= 0 && position <= shortBitLength)
            {               
                OutputValue |= (short)(1 << position);
                bitToSet = position;
            }
        }

        private void ClearBit(short position)
        {
            if (position >= 0 && position <= shortBitLength)
            {
                OutputValue &= (short)~(1 << position);
                bitToClear = position;
            }
        }
    }
}
