﻿using CustomIoTSolution.Common.Config;
using Microsoft.Azure.NotificationHubs;
using Microsoft.Toolkit.Uwp.Notifications;

namespace CustomIoTSolution.EventProcessor.Helpers
{
    public static class NotificationHelper
    {
        private static NotificationHubClient notificationHubClient;

        static NotificationHelper()
        {
            notificationHubClient = NotificationHubClient.CreateClientFromConnectionString(
                Configuration.DefaultFullSharedAccessSignatureConnectionString,
                Configuration.NotificationHubPath);
        }

        public static void SendToast(float temperature)
        {
            // Create toast notification
            var toastContent = new ToastContent()
            {
                Visual = new ToastVisual()
                {                    
                    BindingGeneric = new ToastBindingGeneric()
                    {
                        Children =
                        {
                            new AdaptiveText()
                            {
                                Text = "Temperature alert"
                            },
                            new AdaptiveText()
                            {
                                Text = $"Average temperature of {temperature:F2} exceeded a normal level"
                            }
                        }
                    }
                }
            };

            // Send toast to mobile clients
            notificationHubClient.SendWindowsNativeNotificationAsync(toastContent.GetContent()).Wait();
        }
    }
}
