﻿using CustomIoTSolution.RemoteClient.Helpers;
using CustomIoTSolution.RemoteClient.Models;
using CustomIoTSolution.RemoteClient.TelemetryControl;
using CustomIoTSolution.RemoteClient.ViewModels;
using Microsoft.Azure.Devices.Client;
using System;
using System.Diagnostics;
using System.Threading.Tasks;
using Windows.UI.Core;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;

namespace CustomIoTSolution.RemoteClient
{
    public sealed partial class MainPage : Page
    {
        private const string hostname = "sense-hat-iot-hub.azure-devices.net";
        private const string connectionString = "HostName=sense-hat-iot-hub.azure-devices.net;SharedAccessKeyName=iothubowner;SharedAccessKey=GzTrse4TCH0P/hUCOlIg+moGyeRMydSkRHgSRmLmjts=";
        private const string deviceId = "SenseHAT";

        private ClientViewModel clientViewModel = new ClientViewModel()
        {
            ReadoutDelay = TimeSpan.FromSeconds(10)
        };
       
        private DeviceClient deviceClient;
        
        private Telemetry telemetry;

        public MainPage()
        {
            InitializeComponent();

            CheckDeviceRegistration();
        }

        private async void ButtonRegisterDevice_Click(object sender, RoutedEventArgs e)
        {
            var device = await DeviceRegistrationHelper.RegisterDevice(connectionString, deviceId);

            UpdateDeviceRegistrationDisplay(device.Authentication.SymmetricKey.PrimaryKey);
        }

        private void CheckDeviceRegistration()
        {
            var deviceKey = DeviceRegistrationHelper.RetrieveDeviceKey();

            UpdateDeviceRegistrationDisplay(deviceKey);
        }

        private void UpdateDeviceRegistrationDisplay(string deviceKey)
        {
            // Device is assumed to be registered when the deviceKey is valid
            if (!string.IsNullOrEmpty(deviceKey))
            {
                clientViewModel.IsDeviceRegistered = true;
            }
            else
            {
                clientViewModel.IsDeviceRegistered = false;
            }

            clientViewModel.DeviceKey = deviceKey;
        }

        private async void ButtonRemoveDevice_Click(object sender, RoutedEventArgs e)
        {
            await DeviceRegistrationHelper.RemoveDevice(connectionString, deviceId);

            UpdateDeviceRegistrationDisplay(null);

            if (telemetry != null)
            {
                telemetry.Stop();
            }

            clientViewModel.IsConnected = false;            
        }

        private async void ButtonConnect_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                // Connect to the cloud
                await InitializeDeviceClient();

                // Setup telemetry
                telemetry = await Telemetry.CreateAsync(clientViewModel.ReadoutDelay);
                telemetry.DataReady += Telemetry_DataReady;

                // Update view
                clientViewModel.IsConnected = true;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.Message);
            }
        }

        private void Telemetry_DataReady(object sender, TelemetryEventArgs e)
        {
            DisplaySensorReadings(e);

            var telemetryData = new TelemetryData()
            {
                DeviceId = deviceId,
                Temperature = e.Temperature,
                Humidity = e.Humidity,
                Pressure = e.Pressure
            };

            var telemetryMessage = MessageHelper.Serialize(telemetryData);
            deviceClient.SendEventAsync(telemetryMessage);
        }

        private async void DisplaySensorReadings(TelemetryEventArgs telemetryEventArgs)
        {
            await Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
            {
                clientViewModel.SensorReadings.Temperature = telemetryEventArgs.Temperature;
                clientViewModel.SensorReadings.Humidity = telemetryEventArgs.Humidity;
                clientViewModel.SensorReadings.Pressure = telemetryEventArgs.Pressure;
            });
        }

        private void ButtonStartTelemetry_Click(object sender, RoutedEventArgs e)
        {
            telemetry.Start();
            clientViewModel.IsTelemetryActive = telemetry.IsActive;
        }

        private void ButtonStopTelemetry_Click(object sender, RoutedEventArgs e)
        {
            telemetry.Stop();
            clientViewModel.IsTelemetryActive = telemetry.IsActive;
        }

        private async Task InitializeDeviceClient()
        {
            var authentication = new DeviceAuthenticationWithRegistrySymmetricKey(deviceId, clientViewModel.DeviceKey);

            deviceClient = DeviceClient.Create(hostname, authentication);

            await deviceClient.OpenAsync();
        }
    }
}
