﻿#region Using

using SenseHat.UWP.Sensors;
using System;
using System.ComponentModel;
using System.Runtime.CompilerServices;

#endregion

namespace CustomIoTSolution.RemoteClient.ViewModels
{
    public class ClientViewModel : INotifyPropertyChanged
    {
        #region Public properties

        public string DeviceKey
        {
            get { return deviceKey; }
            set
            {
                deviceKey = value;
                OnPropertyChanged();
            }
        }

        public bool IsDeviceRegistered
        {
            get { return isDeviceRegistered; }
            set
            {
                isDeviceRegistered = value;
                OnPropertyChanged();
            }
        }

        public bool IsConnected
        {
            get { return isConnected; }
            set
            {
                isConnected = value;
                OnPropertyChanged();

                ToggleStartStopButtons(value);
            }
        }

        public bool IsTelemetryActive
        {
            get { return isTelemetryActive; }
            set
            {
                isTelemetryActive = value;
                OnPropertyChanged();

                ToggleStartStopButtons(!value);
            }
        }
        
        public SensorReadings SensorReadings { get; set; } = new SensorReadings();

        // Delay time between consecutive sensor readings
        public TimeSpan ReadoutDelay { get; set; }       

        public bool IsStartTelemetryButtonEnabled { get; private set; }
        public bool IsStopTelemetryButtonEnabled { get; private set; }

        #endregion

        #region Events

        public event PropertyChangedEventHandler PropertyChanged = delegate { };

        #endregion

        #region Fields        

        private bool isDeviceRegistered = false;
        private bool isConnected;
        private bool isTelemetryActive;

        private string deviceKey;

        #endregion

        #region Methods

        private void OnPropertyChanged([CallerMemberName] string propertyName = "")
        {
            PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

        private void ToggleStartStopButtons(bool isStartEnabled)
        {
            IsStartTelemetryButtonEnabled = isStartEnabled && IsConnected;
            OnPropertyChanged("IsStartTelemetryButtonEnabled");

            IsStopTelemetryButtonEnabled = !isStartEnabled && IsConnected;
            OnPropertyChanged("IsStopTelemetryButtonEnabled");
        }

        #endregion
    }
}
